-------------------------------------------------------------------------------
--
-- VGA video generator
--
-- Uses the vga_timing module to generate hsync etc.
-- Massages the RAM address and requests cycles from the memory controller
-- to generate video using one byte per pixel
--
-- Cristian Soviani, Dennis Lim, and Stephen A. Edwards
-- Modified by E. Farhat, E. Herrera, R. Jordan, A. Wilkes
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity vga is
  port (
    clk : in std_logic;
    pix_clk : in std_logic;
    rst : in std_logic;
    video_data : in std_logic_vector(15 downto 0);
    video_addr : out std_logic_vector(19 downto 0);
    video_req : out std_logic;
    vidout_clk : out std_logic;

    blank_len  : in std_logic_vector(15 downto 0);
    start_addr : in std_logic_vector(31 downto 0);
    step_size  : in std_logic_vector(31 downto 0);
    cnt_pix    : in std_logic_vector(15 downto 0);
          
    vidout_RCR : out std_logic_vector(9 downto 0);
    vidout_GY : out std_logic_vector(9 downto 0);
    vidout_BCB : out std_logic_vector(9 downto 0);
    vidout_BLANK_N : out std_logic;
    vidout_HSYNC_N : out std_logic;
    vidout_VSYNC_N : out std_logic);
end vga;

architecture Behavioral of vga is
  
  constant H_ACTIVE             : integer := 480;             
  constant H_FRONT_PORCH        : integer := 96;              
  constant H_BACK_PORCH         : integer := 128;             
  constant H_TOTAL              : integer := 800;             


  -- Fast low-voltage TTL-level I/O pad with 12 mA drive
  
  component OBUF_F_12
    port (
      O : out STD_ULOGIC;
      I : in STD_ULOGIC);
  end component;

  -- Basic edge-sensitive flip-flop
  
  component FD
    port (
      C : in std_logic;
      D : in std_logic;
      Q : out std_logic);
  end component;

  -- Force instances of FD into pads for speed
  
  attribute iob : string;
  attribute iob of FD : component is "true";

  component vga_timing
    port (
      h_sync_delay              : out std_logic;
      v_sync_delay              : out std_logic;
      blank                     : out std_logic;
      vga_ram_read_address      : out std_logic_vector (19 downto 0);
      pixel_clock               : in std_logic;
      reset                     : in std_logic;
      blank_len                 : in std_logic_vector(15 downto 0);
      start_addr                : in std_logic_vector(31 downto 0);
      step_size                 : in std_logic_vector(31 downto 0);
      cnt_pix                   : in std_logic_vector(15 downto 0);     
      pix_cnt                   : out std_logic_vector(10 downto 0)); 
  end component;

  signal r                      : std_logic_vector (9 downto 0);
  signal g                      : std_logic_vector (9 downto 0);
  signal b                      : std_logic_vector (9 downto 0);
  signal blank                  : std_logic;
  signal hsync                  : std_logic;
  signal vsync                  : std_logic;
  signal vga_ram_read_address   : std_logic_vector(19 downto 0);
  signal vreq                   : std_logic;
  signal vreq_1                 : std_logic;
  signal load_video_word        : std_logic;
  signal vga_shreg              : std_logic_vector(15 downto 0);

  signal RAG_counter            : std_logic_vector(19 downto 0);
  signal pixel_count            : std_logic_vector(10 downto 0); --added by Eve
  
begin

  st : vga_timing port map (
    pixel_clock => pix_clk,
    reset => rst,

    h_sync_delay => hsync,
    v_sync_delay => vsync,
    blank => blank,
    vga_ram_read_address => vga_ram_read_address,
    blank_len => blank_len,
    start_addr => start_addr, 
    step_size => step_size,
    cnt_pix => cnt_pix,
    pix_cnt => pixel_count
    );

  vreq <= '1';

  -- Generate load_video_word by delaying vreq two cycles
  
  process (pix_clk)
  begin
    if pix_clk'event and pix_clk='1' then
      vreq_1 <= vreq;
      load_video_word <= vreq_1;
    end if;
  end process;

  -- Generate video_req (to the RAM controller) by delaying vreq by
  -- a cycle synchronized with the pixel clock
  
  process (clk)
  begin
    if clk'event and clk='1' then
      video_req <= pix_clk and vreq;
    end if;
  end process;


  video_addr <= vga_ram_read_address(19 downto 0);

  
  process (pix_clk)
  begin
    if pix_clk'event and pix_clk='1' then
      vga_shreg <= video_data;
    end if;
  end process;
  

  -- RGB 5-6-5
  
  r(9 downto 5) <= vga_shreg (15 downto 11);
  r(4 downto 0) <= "00000";
  g(9 downto 4) <= vga_shreg (10 downto 5);
  g(3 downto 0) <= "0000";
  b(9 downto 5) <= vga_shreg (4 downto 0);
  b(4 downto 0) <= "00000";

  -- Video clock I/O pad to the DAC
  
  vidclk : OBUF_F_12 port map (
    O => VIDOUT_clk,
    I => pix_clk);

  -- Control signals: hsync, vsync, and blank

  hsync_ff : FD port map (
    C => pix_clk,
    D => not hsync,
    Q => VIDOUT_HSYNC_N );

  vsync_ff : FD port map (
    C => pix_clk,
    D => not vsync,
    Q => VIDOUT_VSYNC_N );

  blank_ff :  FD port map (
    C => pix_clk,
    D => not blank,
    Q => VIDOUT_BLANK_N );

  -- Three digital color signals
  
  rgb_ff : for i in 0 to 9 generate
    
    r_ff : FD port map (
      C => pix_clk,
      D => r(i),
      Q => VIDOUT_RCR(i) );
    
    g_ff : FD port map (
      C => pix_clk,
      D => g(i),
      Q => VIDOUT_GY(i) );
    
    b_ff : FD port map (
      C => pix_clk,
      D => b(i),
      Q => VIDOUT_BCB(i) );
    
  end generate;
  
end Behavioral;
